## Claus Hunsen, 2016
## hunsen@fim.uni-passau.de


## libraries
library(argparse) # for ArgumentParser
library(logging) # for the set of log levels


create.cli.parser = function() {

    ## create parser object
    parser = ArgumentParser()

    ## script
    parser$add_argument(
        "script",
        type = "character",
        nargs = 1,
        help = "The subscript to execute"
    )

    ## casestudy
    parser$add_argument(
        "casestudy",
        type = "character",
        nargs = 1,
        help = "Casestudy name as in Codeface-data folder name"
    )

    ## artifact
    parser$add_argument(
        "artifact",
        type = "character",
        nargs = 1,
        help = "Artifact type to use in the analysis"
    )

    ## artifact relation
    parser$add_argument(
        "artifact_relation",
        type = "character",
        nargs = 1,
        help = "Artifact-relation type to use in the analysis"
    )

    ## selection process
    parser$add_argument(
        "-s", "--selection-process",
        type = "character",
        default = "releases",
        help = "The selection process for revision windows [default \"%(default)s\"]"
    )

    ## split window
    parser$add_argument(
        "-w", "--split-window",
        type = "character",
        default = "3 months",
        help = "The time-window length used for data splitting (e.g., \"3 months\" and \"2 weeks\") [default \"%(default)s\"]"
    )

    ## sliding-window approach for split windows?
    parser$add_argument(
        "--sliding-window",
        action = "store_true",
        default = FALSE,
        help = "Do you want overlapping time windows (i.e., sliding-window approach)? [default \"%(default)s\"]"
    )

    ## codeface data
    parser$add_argument(
        "-d", "--codeface-data",
        type = "character",
        default = "/local/hunsen/projects/codeface-data/",
        help = "Path to Codeface data [default \"%(default)s\"]"
    )

    ## null modell to use
    parser$add_argument(
        "--null-model",
        type = "character",
        default = "rewire",
        help = "The specific null model to use ('rewire'|'random'|'errg') [default \"%(default)s\"]"
    )

    ## removal of core devs in null model
    parser$add_argument(
        "--no-remove-core",
        action = "store_true",
        default = FALSE,
        help = "Remove core authors from null model? [default \"%(default)s\"]"
    )

    ## remove core authors in null model?
    parser$add_argument(
        "--complete-rerun",
        action = "store_true",
        default = FALSE,
        help = "Do a complete re-run of the analysis? [default \"%(default)s\"]"
    )

    ## bootstrap packrat (cautious grabbing)
    parser$add_argument(
        "--bootstrap-packrat",
        action = "store_true",
        default = FALSE,
        help = "Bootstrap packrat library?"
    )

    ## log level
    parser$add_argument(
        "--loglevel",
        type = "character",
        default = "INFO",
        help = "Log level"
    )

    return(parser)

}
