## Claus Hunsen, 2019
## hunsen@fim.uni-passau.de

OUTPUT = "png" # one of: pdf, png

## / / / / / / / / / / / / / /
## GLOBAL R CONFIGURATION
##

# disable stringsAsFactors
options(stringsAsFactors = FALSE)
# reset warnings
assign("last.warning", NULL, envir = baseenv())


## / / / / / / / / / / / / / / / /
## Set environment for script ----
##

## source script files
source(file.path("scripts/util-init.R"), chdir = TRUE) # source codeface-extraction-r

source(file.path("util.R")) # get our utility functions
source(file.path("logger.R")) # logging


## / / / / / / / / / / / / / / / /
## CONFIGURATION -----------------
##

## logging
LOG.LEVEL = "DEBUG"
log.folder = file.path("logs-v3", "posthoc")
log.file = file.path(log.folder, "posthoc.log")
log.create(log.level = LOG.LEVEL, log.file = log.file, log.file.remove = TRUE, log.console = TRUE)
logging::loginfo("Initiated logger with log level %s", LOG.LEVEL)

## dummies to load configuration properly
AUTHOR.RELATION = "mail"
ARTIFACT.RELATION = "cochange"

## load configuration
source("configuration.R") # network configuration

## parallelism
library(parallel)
DEBUGGING = Sys.getenv("RSTUDIO") == 1 # set to TRUE for debugging in RStudio
if (DEBUGGING) {
    cores = 1
} else {
    cores = max(4L, detectCores() - 4, na.rm = TRUE)
}
options(mc.cores = cores) # use this number of cores for mclapply


## / / / / / / / / / / / / / / / /
## BusyBox -----------------------
##

# fraction of fulfilled coordination requirements
#                 triangle   square
# Erik Andersen   0.7522936  0.9320975
# Rob Landley     0.5426357  0.7086585
# Denys Vlasenko  0.442577   0.6960127

devs = c("Erik Andersen", "Rob Landley", "Denys Vlasenko")

cr.artifacts = lapply(ARTIFACTS, function(ARTIFACT) {

    folder = sprintf("./data-v3/3_months/busybox/busybox_%s_%s/", ARTIFACT, ARTIFACT.RELATION)

    cr.motifs = lapply(MOTIFS, function(motif) {
        logging::logdebug("%s: %s", ARTIFACT, motif)
        list.motifs.raw = list.files(folder, sprintf("*-motifs.data-raw-%s", motif), full.names = TRUE)

        ranges.active.file = "./data-v3/3_months/busybox/ranges/busybox-ranges.filled"
        load(ranges.active.file)
        list.motifs.raw = list.motifs.raw[ranges.subset.filled]

        cr.ranges = parallel::mclapply(list.motifs.raw, function(motifs.file) {
            logging::logdebug("%s: %s: %s", ARTIFACT, motif, motifs.file)
            net.file = gsub(sprintf("-motifs.data-raw-%s", motif), "-network", motifs.file)

            ## load results
            load(motifs.file)
            ## load corresponding networks (to have names on the vertices!)
            load(net.file)

            # ## retrieve all authors in this range
            # devs = igraph::V(net)[type == TYPE.AUTHOR]$name

            cr.devs = lapply(devs, function(dev) {

                if (length(raw$collaborating) == 0)
                    cr.ident = 0
                else
                    cr.ident = sum(sapply(
                        raw$collaborating,
                        function(x) {dev %in% names(x)}
                    ))

                if (length(setdiff(raw$collaborating, raw$collaborating.and.mailing)) == 0)
                    cr.full = 0
                else
                    cr.full = sum(sapply(
                        setdiff(raw$collaborating, raw$collaborating.and.mailing),
                        function(x) {dev %in% names(x)}
                    ))

                return(data.frame(
                    dev = dev,
                    artifact = ARTIFACT,
                    motif = motif,
                    ident = cr.ident,
                    full = cr.full
                ))
            })

            cr.devs = rbind.fill(cr.devs)
            return(cr.devs)

        })

        cr.ranges = rbind.fill(cr.ranges)
        return(cr.ranges)
    })

    cr.motifs = rbind.fill(cr.motifs)
    return(cr.motifs)
})

cr.artifacts = rbind.fill(cr.artifacts)
a.agg = aggregate(. ~ dev + artifact + motif, data = cr.artifacts, sum)
a.agg["frac"] = a.agg["full"] / a.agg["ident"]

a.cast = dcast(a.agg, dev + artifact ~ motif, value.var = "frac")
a.cast = a.cast[c("dev", "artifact", "triangle", "square")]
a.cast = a.cast[order(match(a.cast$dev, devs)), ]
a.cast = a.cast[order(match(a.cast$artifact, ARTIFACTS)), ]
logging::loginfo(capture.output(print(a.cast)))
