## Claus Hunsen, 2016
## hunsen@fim.uni-passau.de


## / / / / / / / / / / / / / /
## Set environment for script
##

## source script files
source(file.path("util.R")) # get our utility functions
source(file.path("data-motifs.R")) # get motif counts on preprocessed data
source(file.path("data-output.R")) # output data


## / / / / / / / / / / / / / /
## ANALYSIS
##

get.motif.data.for.network = function(net, range) {
    logging::loginfo("Getting motif data %s %s", ANALYSIS, range)

    ## count motifs
    motifs = get.motif.counts(net)
    motifs.data = transform.motif.data.to.data.frame(motifs)
    ## add meta-data
    attr(motifs.data, "range") = range

    return(motifs.data)
}

get.motif.data = function(data) {

  ## compose data and compute metrics for each range
  # data = data[c(1,2)] # FIXME remove this
  motif.counts = mclapply(data, function(range.data) {
      range = range.data$get.range()

      logging::loginfo("Processing %s %s", ANALYSIS, range)

      DUMP.PATH = file.path(TMP.ANALYSIS, paste(ANALYSIS, range, "motifs.data", sep = "-"))
      DUMP.PATH.NETWORK = file.path(TMP.ANALYSIS, paste(ANALYSIS, range, "network", sep = "-"))
      DUMP.PATH.RAW = paste(DUMP.PATH, "raw", "%s", sep = "-")

      motifs.data = load.and.save("motifs.data", DUMP.PATH, skip = RERUN.ANALYSIS, if.not.found = function() {

          ## save the filled range data to disk
          PATH.DATA.FILLED = file.path(TMP.ANALYSIS, paste(ANALYSIS, range, "data", sep = "-"))
          PATH.NET.FILLED = file.path(TMP.ANALYSIS, paste(ANALYSIS, range, "netbuilder", sep = "-"))
          range.data.nets.filled = load.and.save("range.data", PATH.DATA.FILLED, skip = RERUN.ANALYSIS, if.not.found = function() {

              net.builder = load.and.save("net.builder", PATH.NET.FILLED, skip = RERUN.ANALYSIS, if.not.found = function() {
                net.builder = NetworkBuilder$new(range.data, NETCONF)
                net.builder$get.networks()
                return(net.builder)
              })

              return(list(
                  data = range.data,
                  netbuilder = net.builder
              ))
          })
          gc()

          ## construct network
          net = load.and.save("net", DUMP.PATH.NETWORK, skip = RERUN.ANALYSIS, if.not.found = function() {
              net = range.data.nets.filled[["netbuilder"]]$get.multi.network()
              return(net)
          })
          gc()

          motifs.data = get.motif.data.for.network(net, range)
          motifs = names(motifs.data)

          ## store raw data separately!
          motifs.data = lapply(names(motifs.data), function(motif) {
              motif.data = motifs.data[[motif]]
              ## get raw data from attribute
              raw =  attr(motif.data, "raw")
              ## dump raw data
              save(raw, file = sprintf(DUMP.PATH.RAW, motif))
              ## remove the attribute "raw"
              attr(motif.data, "raw") = NULL
              return(motif.data)
          })
          names(motifs.data) = motifs

          ## return motif data
          return(motifs.data)
      })

      logging::loginfo("Finished %s %s", ANALYSIS, range)

      return(motifs.data)
  })

  return(motif.counts)
}


save.data = function(conf, data, motif.counts, base.path = TMP.ANALYSIS, analysis.suffix = NULL) {

  ## save results in global data structure
  results = list(
      "conf" = conf,
      "data" = data,
      "motif.counts" = motif.counts
  )

  analysis.string = ANALYSIS
  if (!is.null(data) && names(data)[1] == "global")
      analysis.string = paste(analysis.string, "global", sep = "-")
  if (!is.null(analysis.suffix))
      analysis.string = paste(analysis.string, analysis.suffix, sep = "-")
  attr(results, "analysis") = analysis.string

  ## dump results to disk
  save(results, file = file.path(base.path, paste(analysis.string, sep = "-")))

  ## combine data sets
  motif.counts.df = combine.data.to.single.data.frame(motif.counts)

  ## dump data to disk
  write.csv(motif.counts.df, file = file.path(base.path, paste0(analysis.string, ".csv")),
            row.names = FALSE, fileEncoding = "utf-8")

}
