set.plot.parameters = function(net, labels = TRUE, grayscale = FALSE) {
    ## colors and shapes
    vertex.colors = c("#00AEFF", "#FF8B00", "#FF0000")
    edge.colors = c("#999999", "#14CC3B", "#FF0000")
    edge.lty = c(2, 1, 3)
    vertex.shapes = c("circle", "square", "triangle")
    vertex.shapes.pch = c(16, 15, 17)

    ## b/w version
    vertex.colors.bw = c("gray40", "gray30", "gray20")
    edge.colors.bw = c("gray60", "gray40")
    if (grayscale) {
        vertex.colors = vertex.colors.bw
        edge.colors = edge.colors.bw
    }

    ## legend colors and types
    net$vertex.colors.legend = c(vertex.colors[1:2], edge.colors[1:2])
    net$vertex.shapes.pch.legend = c(vertex.shapes.pch[1:2], NA, NA)
    net$edge.lty.legend = c(NA, NA, edge.lty[1:2])

    ## vertex color by "type" attribute
    V(net)$color <- vertex.colors[V(net)$type]
    V(net)$shape <- vertex.shapes[V(net)$type]
    # ## vertex color by "community" attribute
    # plot(net, vertex.color = vertex.colors[ V(net)$community ])

    ##  Compute node degrees (#links) and use that to set node size:
    # V(net)$size = degree(net, mode = "all") * 4
    V(net)$label.cex = .8
    V(net)$label.color = ifelse(grayscale, "white", "black")

    ## change arrow size and edge color:
    E(net)$arrow.size <- .2
    # E(net)$width <- log10(1 + E(net)$weight/6)
    # E(net)$curved = .1 # curvy edges
    E(net)$type = unlist(E(net)$type) - 2 # as TYPE.EDGE.INTER is 3, we need to re-map this to 1
    E(net)$color = edge.colors[unlist(E(net)$type)]
    E(net)$lty = edge.lty[unlist(E(net)$type)]

    ## omit labels if wanted
    if (labels == FALSE) {
        V(net)$label = NA
    }

    return(net)
}

## / / / / / / / / / / / / / / / / / / / / / / / / / / / / / /
## Utility function for plotting author networks
##
## Given networks need type attributes on all vertices and edges

plot.sample.network = function(net, labels = TRUE, grayscale = FALSE) {

    ## plot parameters
    net = set.plot.parameters(net, labels = TRUE, grayscale = FALSE)
    V(net)$label.cex = 1.25
    E(net)$width = 2

    ## layout
    lay = matrix(c(  20, 179, 552, 693, 956, 1091, 124, 317, 516, 615, 803, 1038,
                     245, 175, 185, 255, 253, 225,   73,   8,  75,   0,  96,   86),
                 nrow = 12, byrow = FALSE) # for sample graph
    graph_attr(net, "layout") = lay
    # id = tkplot(net, canvas.width = 1450, canvas.height = 450, rescale = FALSE)
    # browser()
    # lay = tk_coords(id)

    ## plot network
    par(mai = c(2,0,0,0), mar = c(0,0,0,0))
    # plot(net, layout = lay, asp = 0, margin = c(0,0,0,0)) # without legend
    plot(net, layout = lay, asp = 0, margin = c(0.4, 0, 0, 0)) # with legend

    # plot legend
    legend("bottom", c("Developers", "Artifacts", "Unipartite Edges", "Bipartite Edges"), bty = "n", ncol = 2,
           pch = net$vertex.shapes.pch.legend, col = net$vertex.colors.legend, pt.bg = net$vertex.colors.legend, lty = net$edge.lty.legend,
           cex = 1.25, pt.cex = 2, lwd = 2)

}


plot.motifs = function(net, labels = TRUE, grayscale = FALSE) {

    ## plot parameters
    net = set.plot.parameters(net, labels = TRUE, grayscale = FALSE)
    V(net)$size = 80
    V(net)$label.cex = 1.2
    E(net)$width = 2
    V(net)$name[1:3] = c("d1", "d2", "a1")
    if (vcount(net) == 4) V(net)$name[4] = "a2"

    ## layout
    types.logical = sapply(V(net)$type, function(x) ifelse(x == TYPE.AUTHOR, FALSE, TRUE))
    lay = layout.bipartite(graph = net, types = types.logical, maxiter = 0)

    ## plot network
    par(mai = c(2,0,0,0), mar = c(0,0,0,0))
    plot(net, layout = lay, asp = 0, margin = c(0.75, 0.2, 0.5, 0.2))

}


plot.example = function(net, labels = TRUE, grayscale = FALSE, layout = NULL) {

    ## plot parameters
    net = set.plot.parameters(net, labels = TRUE, grayscale = FALSE)
    V(net)$size = 12
    V(net)$label.cex = 1
    V(net)$label.degree = sapply(V(net)$type, function(x) ifelse(x == TYPE.AUTHOR, -pi/2, pi/2))
    V(net)$label.dist = 6.75
    E(net)$width = 2

    ## layout
    types.logical = sapply(V(net)$type, function(x) ifelse(x == TYPE.AUTHOR, FALSE, TRUE))
    if (is.null(layout))
        layout = layout.bipartite(graph = net, types = types.logical, maxiter = 0)

    ## plot network
    par(mai = c(2,0,0,0), mar = c(0.75,0,0,0))
    if (get.graph.attribute(net, "artifact") == "function") {
        V(net)$label.dist = sapply(V(net)$type, function(x) ifelse(x == TYPE.AUTHOR, 6.75, 11.25))
        par(mai = c(2,0,0,0), mar = c(0.8,0,0.1,0))
        V(net)$size = 9
    }

    # id = tkplot(net, rescale = FALSE)
    # browser()
    # layout = tk_coords(id)

    plot(net, layout = layout, asp = 0, margin = c(0.75, 0.2, 0.5, 0.2))

}

