## Claus Hunsen, 2016
## hunsen@fim.uni-passau.de

## / / / / / / / / / / / / / /
## GLOBAL R CONFIGURATION
##

# set global mirror for eventually occurring packrat installations
options(repos = list(CRAN="http://cran.rstudio.com/"))
# disable stringsAsFactors
options(stringsAsFactors = FALSE)
# reset warnings
assign("last.warning", NULL, envir = baseenv())


## / / / / / / / / / / / / / /
## Set environment for script
##

## libraries
## logging
library(logging)
logging::logReset()
logging::basicConfig("INFO")
## parallel: make lapply configurable with DEBUGGING
library(parallel)
logging::loginfo("number of logical cores: %s", detectCores())
cores = max(4L, detectCores(), na.rm = TRUE) # set to 1 for disabling of parallelization
options(mc.cores = cores) # use this number of cores for mclapply

## source script files
source(file.path("scripts/util-init.R"), chdir = TRUE) # source codeface-extraction-r

source(file.path("util.R")) # get our utility functions
source(file.path("data-motifs.R")) # get motif counts on preprocessed data
source(file.path("example-plot.R")) # get plotting functions for the sample networks


## NETWORK CONFIGURATION

CF.DATA = "./example"
CF.DATA.PLOTS = file.path(CF.DATA, "plots")
CF.SELECTION.PROCESS = "testing"
CASESTUDY = "test_cr"
AUTHOR.RELATION = "mail"
ARTIFACT.RELATION = "cochange"

dir.create(CF.DATA.PLOTS, recursive = TRUE, showWarnings = FALSE)

source(file.path("configuration.R")) # network configuration


## SAMPLE NETWORKS AND MOTIFS

pdf(file = file.path(CF.DATA.PLOTS, "example-network.pdf"), width = 9, height = 5)
plot.sample.network(get.sample.network())
dev.off()

pdf(file = file.path(CF.DATA.PLOTS, "example-motif-triangle.pdf"), width = 2, height = 2)
plot.motifs(triangle.positive)
dev.off()
svg(file = file.path(CF.DATA.PLOTS, "example-motif-triangle.svg"), width = 2, height = 2)
plot.motifs(triangle.positive)
dev.off()

pdf(file = file.path(CF.DATA.PLOTS, "example-motif-square.pdf"), width = 2, height = 2)
plot.motifs(square.positive)
dev.off()
svg(file = file.path(CF.DATA.PLOTS, "example-motif-square.svg"), width = 2, height = 2)
plot.motifs(square.positive)
dev.off()


## SAMPLE NETWORK FOR CODE EXAMPLE

plot.example.for.artifact = function(artifact, layout = NULL) {
    conf = ProjectConf$new(CF.DATA, CF.SELECTION.PROCESS, CASESTUDY, artifact)
    conf$update.values(PROJCONF.VALUES)
    data = ProjectData$new(conf)
    net.builder = NetworkBuilder$new(data, NETCONF)
    net = net.builder$get.multi.network()
    V(net)$name = gsub("::", "::\n", V(net)$name)
    net = set.graph.attribute(net, "artifact", artifact)
    plot.example(net, layout = layout)
}

pdf(file = file.path(CF.DATA.PLOTS, "example-code-file.pdf"), width = 3, height = 1.25)
plot.example.for.artifact("file", layout = t(matrix(c(0, 100, 100, 100, 200, 100, 150, 0, 50, 0), nrow = 2)))
dev.off()

pdf(file = file.path(CF.DATA.PLOTS, "example-code-function.pdf"), width = 4, height = 1.25)
plot.example.for.artifact("function", layout = t(matrix(c(0, 100, 100, 100, 200, 100, 250, -10, 150, -10, 50, -10, -50, -10), nrow = 2)))
dev.off()

pdf(file = file.path(CF.DATA.PLOTS, "example-code-feature.pdf"), width = 3, height = 1.25)
plot.example.for.artifact("feature", layout = t(matrix(c(0, 100, 100, 100, 200, 100, 150, 0, 50, 0), nrow = 2)))
dev.off()
