## Claus Hunsen, 2017
## hunsen@fim.uni-passau.de


## / / / / / / / / / / / / / /
## Set environment for script
##

## libraries
library(logging)

## create path for logs
LOG.PATH = file.path("logs")
dir.create(LOG.PATH, recursive = TRUE, showWarnings = FALSE)


## / / / / / / / / / / / / / /
## LOGGING
##

## log format
log.format <- function(record) {
    text = paste(record$timestamp, "Task", Sys.getpid(), "-", paste(record$levelname, record$logger, record$msg, sep=':'))
    return(text)
}

# ## create default logger
log.create = function(log.level, log.file = NULL, log.file.remove = TRUE, log.console = TRUE) {
    require(logging)

    ## reset everything
    logging::logReset()

    ## log to file
    if (!is.null(log.file)) {
        ## remove file if wanted
        if (log.file.remove && file.exists(log.file)) {
            file.remove(log.file)
        }
        ## create base folder
        log.dir = dirname(log.file)
        dir.create(log.dir, recursive = TRUE, showWarnings = FALSE)

        ## add writeToFile handler
        logging::addHandler(logging::writeToFile, file = log.file, level = log.level, formatter = log.format)
    }


    ## add writeToConsole handler if wanted
    if (is.null(log.file)) log.console = TRUE # log to one sink, at least
    if (log.console) {
        logging::addHandler(logging::writeToConsole, level = log.level, formatter = log.format)
    }

    ## set log level explicitly
    logging::setLevel(log.level)
}
